package eu.ztsh.garmin

import android.annotation.SuppressLint
import android.location.Location
import android.util.Log
import androidx.lifecycle.DefaultLifecycleObserver
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LifecycleOwner
import com.mapbox.navigation.base.formatter.DistanceFormatterOptions
import com.mapbox.navigation.base.options.NavigationOptions
import com.mapbox.navigation.core.formatter.MapboxDistanceFormatter
import com.mapbox.navigation.core.lifecycle.MapboxNavigationApp
import com.mapbox.navigation.core.trip.session.LocationMatcherResult
import com.mapbox.navigation.core.trip.session.LocationObserver
import com.mapbox.navigation.core.trip.session.RouteProgressObserver
import com.mapbox.navigation.ui.maneuver.api.MapboxManeuverApi

@SuppressLint("MissingPermission")
class MapboxToolbox(lifecycle: Lifecycle, private val context: MainActivity) {

    private val mapboxObserver = MapboxObserver()

    init {
        lifecycle.addObserver(object : DefaultLifecycleObserver {
            override fun onResume(owner: LifecycleOwner) {
                MapboxNavigationApp.attach(owner)
                MapboxNavigationApp.registerObserver(mapboxObserver)
            }

            override fun onPause(owner: LifecycleOwner) {
                MapboxNavigationApp.detach(owner)
                MapboxNavigationApp.unregisterObserver(mapboxObserver)
            }
        })
    }

    fun onCreate() {
        if (!MapboxNavigationApp.isSetup()) {
            MapboxNavigationApp.setup {
                NavigationOptions.Builder(context)
                    .accessToken(BuildConfig.MAPBOX_DOWNLOADS_TOKEN)
                    .build()
            }
        }
        MapboxNavigationApp.current()?.startTripSession()
    }

    fun onStart() {
        MapboxNavigationApp.current()?.registerRouteProgressObserver(routeProgressObserver)
        MapboxNavigationApp.current()?.registerLocationObserver(locationObserver)
//        MapboxNavigationApp.current()?.registerNavigationSessionStateObserver()
    }

    fun onStop() {
        MapboxNavigationApp.current()?.unregisterRouteProgressObserver(routeProgressObserver)
        MapboxNavigationApp.current()?.unregisterLocationObserver(locationObserver)
    }

    fun onDestroy() {
        MapboxNavigationApp.current()?.stopTripSession()
        maneuverApi.cancel()
    }

    // Define distance formatter options
    private val distanceFormatter: DistanceFormatterOptions by lazy {
        DistanceFormatterOptions.Builder(context).build()
    }
    // Create an instance of the Maneuver API
    private val maneuverApi: MapboxManeuverApi by lazy {
        MapboxManeuverApi(MapboxDistanceFormatter(distanceFormatter))
    }

    private val routeProgressObserver =
        RouteProgressObserver { routeProgress ->
            maneuverApi.getManeuvers(routeProgress).value?.apply {
                context.garmin.process(
                    this[0]
                )
            }
        }

    private val locationObserver = object : LocationObserver {
        override fun onNewLocationMatcherResult(locationMatcherResult: LocationMatcherResult) {
            Log.d("LOCATION", "")
        }

        override fun onNewRawLocation(rawLocation: Location) {
        }
    }

}